@@ -2,8 +2,8 @@
 // SPDX-License-Identifier: Apache-2.0
 use super::utils::to_response_status;
 use super::{utils, KeyPairType, Provider};
-use crate::authenticators::ApplicationName;
-use crate::key_info_managers::KeyTriple;
+use crate::authenticators::ApplicationIdentity;
+use crate::key_info_managers::KeyIdentity;
 use cryptoki::types::mechanism::{Mechanism, MechanismType};
 use cryptoki::types::object::{Attribute, AttributeType, KeyType, ObjectClass, ObjectHandle};
 use cryptoki::types::session::Session;
@@ -12,10 +12,10 @@ use parsec_interface::operations::psa_key_attributes::{EccFamily, Id, Lifetime,
 use parsec_interface::operations::{
     psa_destroy_key, psa_export_public_key, psa_generate_key, psa_import_key,
 };
-use parsec_interface::requests::{ProviderId, ResponseStatus, Result};
+use parsec_interface::requests::{ResponseStatus, Result};
 use parsec_interface::secrecy::ExposeSecret;
 use picky_asn1::wrapper::{IntegerAsn1, OctetStringAsn1};
-use picky_asn1_x509::RSAPublicKey;
+use picky_asn1_x509::RsaPublicKey;
 use std::convert::{TryFrom, TryInto};
 
 impl Provider {
@@ -47,16 +47,16 @@ impl Provider {
         }
     }
 
-    pub(super) fn move_pub_key_to_psa_crypto(&self, key_triple: &KeyTriple) -> Result<Id> {
+    pub(super) fn move_pub_key_to_psa_crypto(&self, key_identity: &KeyIdentity) -> Result<Id> {
         info!("Attempting to export public key");
         let export_operation = psa_export_public_key::Operation {
-            key_name: key_triple.key_name().to_owned(),
+            key_name: key_identity.key_name().to_owned(),
         };
         let psa_export_public_key::Result { data } =
-            self.psa_export_public_key_internal(key_triple.app_name().clone(), export_operation)?;
+            self.psa_export_public_key_internal(key_identity.application(), export_operation)?;
 
         info!("Importing public key into PSA Crypto");
-        let mut attributes = self.key_info_store.get_key_attributes(&key_triple)?;
+        let mut attributes = self.key_info_store.get_key_attributes(&key_identity)?;
         attributes.lifetime = Lifetime::Volatile;
         attributes.key_type = match attributes.key_type {
             Type::RsaKeyPair | Type::RsaPublicKey => Type::RsaPublicKey,
@@ -84,7 +84,7 @@ impl Provider {
 
     pub(super) fn psa_generate_key_internal(
         &self,
-        app_name: ApplicationName,
+        application_identity: &ApplicationIdentity,
         op: psa_generate_key::Operation,
     ) -> Result<psa_generate_key::Result> {
         if op.attributes.key_type.is_public_key() {
@@ -100,8 +100,12 @@ impl Provider {
             return Err(ResponseStatus::PsaErrorNotPermitted);
         }
 
-        let key_triple = KeyTriple::new(app_name, ProviderId::Pkcs11, key_name);
-        self.key_info_store.does_not_exist(&key_triple)?;
+        let key_identity = KeyIdentity::new(
+            application_identity.clone(),
+            self.provider_identity.clone(),
+            key_name,
+        );
+        self.key_info_store.does_not_exist(&key_identity)?;
 
         let session = self.new_session()?;
 
@@ -152,7 +156,7 @@ impl Provider {
             Ok((public, private)) => {
                 if let Err(e) =
                     self.key_info_store
-                        .insert_key_info(key_triple, &key_id, key_attributes)
+                        .insert_key_info(key_identity, &key_id, key_attributes)
                 {
                     format_error!("Failed to insert the mappings, deleting the key", e);
                     if let Err(e) = session.destroy_object(public) {
@@ -175,7 +179,7 @@ impl Provider {
 
     pub(super) fn psa_import_key_internal(
         &self,
-        app_name: ApplicationName,
+        application_identity: &ApplicationIdentity,
         op: psa_import_key::Operation,
     ) -> Result<psa_import_key::Result> {
         let key_name = op.key_name;
@@ -185,9 +189,17 @@ impl Provider {
             error!("The configuration of this provider does not allow it to generate keys that can be exported.");
             return Err(ResponseStatus::PsaErrorNotPermitted);
         }
+        if op.data.expose_secret().is_empty() {
+            error!("Key data is empty");
+            return Err(ResponseStatus::PsaErrorInvalidArgument);
+        }
 
-        let key_triple = KeyTriple::new(app_name, ProviderId::Pkcs11, key_name);
-        self.key_info_store.does_not_exist(&key_triple)?;
+        let key_identity = KeyIdentity::new(
+            application_identity.clone(),
+            self.provider_identity.clone(),
+            key_name,
+        );
+        self.key_info_store.does_not_exist(&key_identity)?;
 
         let session = self.new_session()?;
 
@@ -228,7 +240,7 @@ impl Provider {
             Ok(key) => {
                 if let Err(e) =
                     self.key_info_store
-                        .insert_key_info(key_triple, &key_id, key_attributes)
+                        .insert_key_info(key_identity, &key_id, key_attributes)
                 {
                     format_error!("Failed to insert the mappings, deleting the key.", e);
                     if let Err(e) = session.destroy_object(key) {
@@ -252,8 +264,8 @@ impl Provider {
         bits: usize,
         template: &mut Vec<Attribute>,
     ) -> Result<()> {
-        let public_key: RSAPublicKey = picky_asn1_der::from_bytes(key_data).map_err(|e| {
-            format_error!("Failed to parse RSAPublicKey data", e);
+        let public_key: RsaPublicKey = picky_asn1_der::from_bytes(key_data).map_err(|e| {
+            format_error!("Failed to parse RsaPublicKey data", e);
             ResponseStatus::PsaErrorInvalidArgument
         })?;
 
@@ -306,6 +318,24 @@ impl Provider {
             }
             _ => (),
         }
+
+        // For the format of ECC public keys, see:
+        // https://parallaxsecond.github.io/parsec-book/parsec_client/operations/psa_export_public_key.html#description
+        let key_len = ((key_data.len() - 1) / 2) * 8;
+        let bits = if bits == 0 { key_len } else { bits };
+        if bits != key_len {
+            if crate::utils::GlobalConfig::log_error_details() {
+                error!(
+                        "`bits` field of key attributes (value: {}) must be either 0 or equal to half the size of the key in `data` (value: {}) for Weierstrass curves.",
+                        bits,
+                        key_len
+                    );
+            } else {
+                error!("`bits` field of key attributes must be either 0 or equal to half the size of the key in `data` for Weierstrass curves.");
+            }
+            return Err(ResponseStatus::PsaErrorInvalidArgument);
+        }
+
         // The format expected by PKCS11 is an ASN.1 OctetString containing the
         // data that the PSA Crypto interface specifies.
         // See ECPoint in [SEC1](https://www.secg.org/sec1-v2.pdf). PKCS11 mandates using
@@ -332,13 +362,17 @@ impl Provider {
 
     pub(super) fn psa_export_public_key_internal(
         &self,
-        app_name: ApplicationName,
+        application_identity: &ApplicationIdentity,
         op: psa_export_public_key::Operation,
     ) -> Result<psa_export_public_key::Result> {
         let key_name = op.key_name;
-        let key_triple = KeyTriple::new(app_name, ProviderId::Pkcs11, key_name);
-        let key_attributes = self.key_info_store.get_key_attributes(&key_triple)?;
-        let key_id = self.key_info_store.get_key_id(&key_triple)?;
+        let key_identity = KeyIdentity::new(
+            application_identity.clone(),
+            self.provider_identity.clone(),
+            key_name,
+        );
+        let key_attributes = self.key_info_store.get_key_attributes(&key_identity)?;
+        let key_id = self.key_info_store.get_key_id(&key_identity)?;
         let session = self.new_session()?;
 
         let key = self.find_key(&session, key_id, KeyPairType::PublicKey)?;
@@ -384,7 +418,7 @@ impl Provider {
             return Err(ResponseStatus::PsaErrorCommunicationFailure);
         };
 
-        let key = RSAPublicKey {
+        let key = RsaPublicKey {
             modulus,
             public_exponent,
         };
@@ -421,14 +455,18 @@ impl Provider {
 
     pub(super) fn psa_destroy_key_internal(
         &self,
-        app_name: ApplicationName,
+        application_identity: &ApplicationIdentity,
         op: psa_destroy_key::Operation,
     ) -> Result<psa_destroy_key::Result> {
         let key_name = op.key_name;
-        let key_triple = KeyTriple::new(app_name, ProviderId::Pkcs11, key_name);
-        let key_id = self.key_info_store.get_key_id(&key_triple)?;
+        let key_identity = KeyIdentity::new(
+            application_identity.clone(),
+            self.provider_identity.clone(),
+            key_name,
+        );
+        let key_id = self.key_info_store.get_key_id(&key_identity)?;
 
-        let _ = self.key_info_store.remove_key_info(&key_triple)?;
+        let _ = self.key_info_store.remove_key_info(&key_identity)?;
 
         let session = self.new_session()?;
 
