@@ -9,6 +9,7 @@
 
 #include <psa/error.h>
 #include <rpc_caller.h>
+#include <service/common/client/service_client.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -18,13 +19,12 @@ extern "C" {
  * @brief      The singleton psa_crypto_client state
  *
  * The psa crypto C API assumes a single instance of the backend provider.  This
- * structure defines the state used by the psa_cryypto_client that communicates
- * with a remote provider using the provided rpc caller.
+ * structure extends the base service client.
  */
 struct psa_crypto_client
 {
-    struct rpc_caller *caller;
-    int rpc_status;
+	struct service_client base;
+	psa_status_t init_status;
 };
 
 extern struct psa_crypto_client psa_crypto_client_instance;
@@ -57,6 +57,19 @@ void psa_crypto_client_deinit(void);
  */
 int psa_crypto_client_rpc_status(void);
 
+/**
+ * @brief      Get the base service_client
+ *
+ * Returns a pointer to the base service_client member of the singleton psa crypto
+ * client.
+ *
+ * @return     Base service_client
+ */
+static inline struct service_client *psa_crypto_client_base(void)
+{
+	return &psa_crypto_client_instance.base;
+}
+
 #ifdef __cplusplus
 }
 #endif
