@@ -10,8 +10,12 @@
 #include <protocols/service/crypto/packed-c/key_attributes.h>
 #include <psa/crypto.h>
 
-void packedc_crypto_provider_translate_key_attributes(
+void packedc_crypto_provider_translate_key_attributes_from_proto(
     psa_key_attributes_t *psa_attributes,
     const struct ts_crypto_key_attributes *proto_attributes);
 
-#endif /* PACKEDC_CRYPTO_PROVIDER_KEY_ATTRIBUTES_TRANSLATOR_H */
\ No newline at end of file
+void packedc_crypto_provider_translate_key_attributes_to_proto(
+    struct ts_crypto_key_attributes *proto_attributes,
+    const psa_key_attributes_t *psa_attributes);
+
+#endif /* PACKEDC_CRYPTO_PROVIDER_KEY_ATTRIBUTES_TRANSLATOR_H */
