@@ -9,7 +9,7 @@
 
 #include <service/locator/standalone/standalone_service_context.h>
 #include <rpc/direct/direct_caller.h>
-#include <service/crypto/provider/mbedcrypto/crypto_provider.h>
+#include <service/crypto/provider/crypto_provider.h>
 #include <service/secure_storage/backend/secure_storage_client/secure_storage_client.h>
 #include <service/secure_storage/backend/null_store/null_store.h>
 
@@ -24,7 +24,7 @@ private:
     void do_init();
     void do_deinit();
 
-    struct mbed_crypto_provider m_crypto_provider;
+    struct crypto_provider *m_crypto_provider;
     struct secure_storage_client m_storage_client;
     struct null_store m_null_store;
     struct service_context *m_storage_service_context;
