@@ -7,6 +7,7 @@
 #ifndef STANDALONE_SERVICE_CONTEXT_H
 #define STANDALONE_SERVICE_CONTEXT_H
 
+#include <cstddef>
 #include <service_locator.h>
 #include <rpc/common/endpoint/rpc_interface.h>
 #include <rpc/direct/direct_caller.h>
@@ -16,6 +17,7 @@ class standalone_service_context
 {
 public:
     standalone_service_context(const char *sn);
+    standalone_service_context(const char *sn, size_t rpc_buffer_size_override);
     virtual ~standalone_service_context();
 
     void init();
@@ -37,7 +39,8 @@ private:
 
     struct rpc_session
     {
-        rpc_session(struct rpc_interface *rpc_interface);
+        rpc_session(struct rpc_interface *rpc_interface,
+            size_t rpc_buffer_size_override);
         ~rpc_session();
 
         struct direct_caller m_direct_caller;
@@ -46,6 +49,7 @@ private:
 
     std::string m_sn;
     int m_ref_count;
+    size_t m_rpc_buffer_size_override;
     struct service_context m_service_context;
     struct rpc_interface *m_rpc_interface;
 };
