@@ -5,7 +5,7 @@
 
 #include <service/test_runner/provider/backend/simple_c/simple_c_test_runner.h>
 #include <config/test/sp/sp_config_tests.h>
-#include <service/crypto/provider/mbedcrypto/trng_adapter/test/trng_env_tests.h>
+#include <service/crypto/backend/mbedcrypto/trng_adapter/test/trng_env_tests.h>
 
 void env_test_register_tests(struct test_runner_provider *context)
 {
@@ -13,4 +13,4 @@ void env_test_register_tests(struct test_runner_provider *context)
 
 	sp_config_tests_register();
 	trng_env_tests_register();
-}
\ No newline at end of file
+}
