@@ -10,7 +10,9 @@
 /**
  *  C/C++ definition of test_runner service opcodes
  */
-#define TS_TEST_RUNNER_OPCODE_RUN_TESTS           (0x0101)
-#define TS_TEST_RUNNER_OPCODE_LIST_TESTS          (0x0102)
+
+#define TS_TEST_RUNNER_OPCODE_BASE              (0x0100)
+#define TS_TEST_RUNNER_OPCODE_RUN_TESTS         (TS_TEST_RUNNER_OPCODE_BASE + 1)
+#define TS_TEST_RUNNER_OPCODE_LIST_TESTS        (TS_TEST_RUNNER_OPCODE_BASE + 2)
 
 #endif /* TS_TEST_RUNNER_OPCODES_H */
